@@ -353,7 +353,7 @@ type_names['FILE*'] = TYPE_ANY
 # integers it's easy enough to expand them.
 type_names['size_t'] = type_names['gsize']
 type_names['ssize_t'] = type_names['gssize']
-type_names['time_t'] = TYPE_INT64
+type_names['time_t'] = TYPE_LONG
 type_names['off_t'] = type_names['gsize']
 type_names['pid_t'] = TYPE_INT
 type_names['uid_t'] = TYPE_UINT
@@ -669,6 +669,11 @@ GIName.  It's possible for nodes to contain or point to other nodes."""
         pass
 
 
+class DocSection(Node):
+    def __init__(self, name=None):
+        Node.__init__(self, name)
+
+
 class Registered:
     """A node that (possibly) has gtype_name and get_type."""
     def __init__(self, gtype_name, get_type):
