@@ -18,17 +18,11 @@
 # Boston, MA 02111-1307, USA.
 #
 
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-from __future__ import unicode_literals
-
 import os
 import sys
 import tempfile
-import shutil
 import subprocess
-from xml.etree.cElementTree import parse
+from xml.etree.ElementTree import parse
 
 from . import ast
 from . import message
@@ -89,12 +83,12 @@ class GDumpParser(object):
         """
 
         # First pass: parsing
-        for node in self._namespace.values():
+        for node in list(self._namespace.values()):
             if isinstance(node, ast.Function):
                 self._initparse_function(node)
 
         if self._namespace.name == 'GObject' or self._namespace.name == 'GLib':
-            for node in self._namespace.values():
+            for node in list(self._namespace.values()):
                 if isinstance(node, ast.Record):
                     self._initparse_gobject_record(node)
 
@@ -123,7 +117,7 @@ class GDumpParser(object):
         # Pair up boxed types and class records
         for name, boxed in self._boxed_types.items():
             self._pair_boxed_type(boxed)
-        for node in self._namespace.values():
+        for node in list(self._namespace.values()):
             if isinstance(node, (ast.Class, ast.Interface)):
                 self._find_class_record(node)
 
@@ -150,7 +144,7 @@ class GDumpParser(object):
         """Load the library (or executable), returning an XML
 blob containing data gleaned from GObject's primitive introspection."""
         in_path = os.path.join(self._binary.tmpdir, 'functions.txt')
-        with open(in_path, 'w') as f:
+        with open(in_path, 'w', encoding='utf-8') as f:
             for func in self._get_type_functions:
                 f.write('get-type:')
                 f.write(func)
@@ -181,7 +175,7 @@ blob containing data gleaned from GObject's primitive introspection."""
             return parse(out_path)
         finally:
             if not utils.have_debug_flag('save-temps'):
-                shutil.rmtree(self._binary.tmpdir)
+                utils.rmtree(self._binary.tmpdir)
 
     # Parser
 
